/**
 *  @filename   :   epd7in5b.cpp
 *  @brief      :   Implements for e-paper library
 *  
 *  based on Waveshare Demo Software for Arduino and STM32
 *  
 */

#include <stdlib.h>
#include "epd7in5b_HD.h"

Epd::~Epd() 
{
};

Epd::Epd() 
{
    reset_pin = RST_PIN;
    dc_pin    = DC_PIN;
    cs_pin    = CS_PIN;
    busy_pin  = BUSY_PIN;
    width     = EPD_WIDTH;
    height    = EPD_HEIGHT;
};

int Epd::Init(void) 
{
    if (IfInit() != 0) 
    {
        return -1;
    }
    Reset();
    
    SendCommand(SW_RESET);
    WaitUntilIdle();

    SendCommand(AUTO_WRITE_RED_RAM);
    SendData(0xF7);
    WaitUntilIdle();

    SendCommand(AUTO_WRITE_BW_RAM);
    SendData(0xF7);
    WaitUntilIdle();

    SendCommand(0x0C);  // Soft start setting
    SendData(0xAE);
    SendData(0xC7);
    SendData(0xC3);
    SendData(0xC0);
    SendData(0x40);   

    SendCommand(GATE_SETTING);  // Set MUX as 527
    SendData(0xAF);
    SendData(0x02);
    SendData(0x01);

    SendCommand(DATA_ENTRY_SEQUENCE);
    SendData(0x01);

    SendCommand(SET_RAM_X_POS_START_END);
    SendData(0x00);    // RAM x address start at 0
    SendData(0x00);
    SendData(0x6F);    // RAM x address end at 36Fh -> 879
    SendData(0x03);
    SendCommand(SET_RAM_Y_POS_START_END);
    SendData(0xAF);    // RAM y address start at 20Fh -> 527
    SendData(0x02);
    SendData(0x00);    // RAM y address end at 00h;
    SendData(0x00);

    SendCommand(BORDER_WAVEFORM_CONTROL); // VBD
    SendData(0x01);    // LUT1, for white

    SendCommand(TEMPERATURE_SENSOR_SELECTION);
    SendData(0X80);
    SendCommand(DISPLAY_UPDATE_CONTROL2);
    SendData(0XB1);	   // Load Temperature and waveform setting.
    SendCommand(ACTIVATE_DISPLAY_UPDATE_SEQUENCE);
    WaitUntilIdle();   // waiting for the electronic paper IC to release the idle signal

    SendCommand(SET_RAM_X_ADDRESS_COUNTER); 
    SendData(0x00);
    SendData(0x00);
    SendCommand(SET_RAM_Y_ADDRESS_COUNTER); 
    SendData(0xAF);
    SendData(0x02);

    return 0;
}

/**
 *  @brief: basic function for sending commands
 */
void Epd::SendCommand(unsigned char command) 
{
    DigitalWrite(dc_pin, LOW);
    SpiTransfer(command);
}

/**
 *  @brief: basic function for sending data
 */
void Epd::SendData(unsigned char data) 
{
    DigitalWrite(dc_pin, HIGH);
    SpiTransfer(data);
}

/**
 *  @brief: Wait until the busy_pin goes HIGH
 
 */
void Epd::WaitUntilIdle(void) 
{
    unsigned char busy;
    while(DigitalRead(busy_pin))
    {
        // DelayMs(10);   
    }
    DelayMs(200);      
}

/******************************************************************************
function :  Clear screen
parameter:
******************************************************************************/
void Epd::Clear(void)
{
    uint32_t i;

    SendCommand(SET_RAM_Y_ADDRESS_COUNTER); 
    SendData(0x00);
    SendData(0x00);

    SendCommand(WRITE_RAM_BW);
    for(i = 0; i < 58080 + 1; i++) 
    {
        SendData(0xff);
    }
 
    SendCommand(WRITE_RAM_RED);
    for(i = 0; i < 58080; i++)
    {
        SendData(0x00);
    }
    
    SendCommand(DISPLAY_UPDATE_CONTROL2);
    SendData(0xF7);//Load LUT from MCU(0x32)
    SendCommand(ACTIVATE_DISPLAY_UPDATE_SEQUENCE);
    DelayMs(10);
    WaitUntilIdle();
}

/**
 *  @brief: module reset.
 *          often used to awaken the module in deep sleep,
 *          see Epd::Sleep();
 */
void Epd::Reset(void) 
{
    DigitalWrite(reset_pin, LOW);                //module reset    
    DelayMs(2);
    DigitalWrite(reset_pin, HIGH);
    DelayMs(200);
}

/******************************************************************************
function :  Turn On Display
parameter:
******************************************************************************/
void Epd::TurnOnDisplay(void)
{
    SendCommand(DISPLAY_UPDATE_CONTROL2);
    SendData(0xC7);    //Load LUT from MCU(0x32)
    
    SendCommand(ACTIVATE_DISPLAY_UPDATE_SEQUENCE);
    
    DelayMs(10);       //!!!The delay here is necessary, 200uS at least!!!     
    WaitUntilIdle();   //waiting for the electronic paper IC to release the idle signal
}

void Epd::WriteBlackImage(void)
{
    SendCommand(SET_RAM_Y_ADDRESS_COUNTER); 
    SendData(0x00);
    SendData(0x00);
    SendCommand(WRITE_RAM_BW);
}

void Epd::WriteRedImage(void)
{
    SendCommand(SET_RAM_Y_ADDRESS_COUNTER); 
    SendData(0x00);
    SendData(0x00);
    SendCommand(WRITE_RAM_RED);
}

/******************************************************************************
function :  Sends the image buffer in Flash to e-Paper and displays
parameter:  image_width and start_X :Must be a multiple of 8
******************************************************************************/
void Epd::DisplayBlackImage(const uint8_t *blackimage,uint32_t start_X, uint32_t start_Y, uint32_t image_width, uint32_t image_high)
{
    uint32_t  Width, Height;
    uint32_t i,j;
    
    Width  = (EPD_WIDTH % 8 == 0)?(EPD_WIDTH / 8 ):(EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    WriteBlackImage();
    for (j = 0; j < Height; j++) 
    {
        for (i = 0; i < Width; i++) 
        {
            if((i >= start_X / 8) && (i < (start_X + image_width) / 8) && (j >= start_Y) && (j < (start_Y + image_high)))
            {
              SendData(~pgm_read_byte(&blackimage[i - start_X / 8 + (j - start_Y) * image_width / 8]));
            }else
            {
              SendData(0xff);
            }
        }
    }
    TurnOnDisplay();
}

void Epd::DisplayRedImage(const uint8_t *image,uint32_t start_X, uint32_t start_Y, uint32_t image_width, uint32_t image_high)
{
    uint32_t  Width, Height;
    uint32_t i,j;
    
    Width  = (EPD_WIDTH % 8 == 0)?(EPD_WIDTH / 8 ):(EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    WriteRedImage();
    for (j = 0; j < Height; j++) 
    {
        for (i = 0; i < Width; i++) 
        {
            if((i >= start_X / 8) && (i < (start_X + image_width) / 8) && (j >= start_Y) && (j < (start_Y + image_high)))
            {
              SendData(pgm_read_byte(&image[i - start_X / 8 + (j - start_Y) * image_width / 8]));
            }else
            {
              SendData(0xff);
            }
        }
    }
    TurnOnDisplay();
}

/******************************************************************************
function :  Sends the image buffer in Flash to e-Paper and displays
parameter:
******************************************************************************/
void Epd::Display(const uint8_t *blackimage)
{
    uint32_t Width, Height;
    uint32_t i,j;
    
    Width  = (EPD_WIDTH % 8 == 0)?(EPD_WIDTH / 8 ):(EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    SendCommand(SET_RAM_Y_ADDRESS_COUNTER); 
    SendData(0x00);
    SendData(0x00);

    SendCommand(WRITE_RAM_BW);
    for (j = 0; j < Height; j++) 
    {
        for (i = 0; i < Width; i++) 
        {
            SendData(pgm_read_byte(&blackimage[i + j * Width]));
        }
    }
    
    SendCommand(WRITE_RAM_RED);
    for(i = 0; i < Height * Width; i++) 
    {
        SendData(0x00);
    }
    TurnOnDisplay();
}

/******************************************************************************
function :  Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void Epd::DisplayPictureFromRAM(const uint8_t *image, uint8_t Block)
{
  uint32_t Width, Height;
  uint32_t i,j;
  
  Width  = (EPD_WIDTH % 8 == 0)?(EPD_WIDTH / 8 ):(EPD_WIDTH / 8 + 1);
  Height = EPD_HEIGHT2;
  
  SendCommand(SET_RAM_Y_ADDRESS_COUNTER); 
  SendData(0x00);
  SendData(0x00);

  switch(Block)
  {
    case 0: // black
    SendCommand(WRITE_RAM_BW);
    for (j = 0; j < Height; j++) 
    {
      for (i = 0; i < Width; i++) 
      {
        SendData(image[i + j * Width]);
      }
    }
      break;

    case 1: // red
    SendCommand(WRITE_RAM_RED);
    for (j = 0; j < Height; j++) 
    {
      for (i = 0; i < Width; i++) 
      {
        SendData(~image[i + j * Width]);
      }
    }
      break;
    default:
      break;
  }
  TurnOnDisplay();
}

/**
 *  @brief: After this command is transmitted, the chip would enter the 
 *          deep-sleep mode to save power. 
 *          The deep sleep mode would return to standby by hardware reset. 
 *          The only one parameter is a check code, the command would be
 *          executed if check code = 0xA5. 
 *          You can use EPD_Reset() to awaken
 */
void Epd::Sleep(void) 
{
    SendCommand(DEEP_SLEEP_MODE_CONTROL);
    SendData(0x01);
}

/* END OF FILE */
